@@ -5,7 +5,7 @@ from pycape import Cape
 from pycape import FunctionRef
 
 if __name__ == "__main__":
-    url = os.environ.get("CAPE_HOST", "wss://enclave.capeprivacy.com")
+    url = os.environ.get("CAPE_HOST", "https://app.capeprivacy.com")
     function_json = os.environ.get("FUNCTION_JSON", "echo_token.json")
     function_json = pathlib.Path(__file__).parent.absolute() / function_json
 
